

/* 
Rule format:
 one of:
AbstractSyntaxExpr -> Triples.
AbstractSyntaxExpr -> Triples; MainNode .
SubjNode+AbstractSyntax -> Triples.


Internal syntax
   id-subscript
   note(link)/thingy
*/

:-op(150,xfy,@).
:-op(150,xfy,^^).

text -> heading(1,'URI References').
text2 -> 'The following rules are ones that use a node provided from higher
 in the abstract syntax tree. This node is shown as <code><strong>@</strong></code>.
It is always the subject of the triples.'.

o(O) -> 
        t(O,rdf:type,owl:ontology);      O.
dt(DT) -> 
         t(DT,rdf:type,rdfs:datatype);  DT.

dt(rdf:xMLLiteral) -> true;              rdf:xMLLiteral.

dt(xsd:Builtin) -> true;                 xsd:Builtin.

dt(Any) ->
         t(Any, rdf:type, rdfs:datatype ); Any.


c(owl:thing) -> true;                    owl:thing.
c(owl:nothing) -> true;                  owl:nothing.
c(C) -> 
         t(C,rdf:type,owl:class),
         +[t(C,rdf:type,rdfs:class)];     C.

i(I) -> true;                            I.

m(owl:Builtin) -> true;                  owl:Builtin.
m(rdfs:seeAlso) -> true;                 rdfs:seeAlso .
m(rdfs:isDefinedBy) -> true;             rdfs:isDefinedBy.
dr(rdfs:literal) -> true;                rdfs:literal.
dr(DR) -> 
         note(dataRange)/t(DR,rdf:type,owl:dataRange);   DR.

sdp(DP) -> 
          t(DP,rdf:type,owl:datatypeProperty),
          +[t(DP,rdf:type,rdf:property)];
                                          DP.

sdp(rdfs:comment) -> true;    rdfs:comment .
sdp(rdfs:label) -> true;      rdfs:label .

cdp(DP) -> 
          t(DP,rdf:type,owl:datatypeProperty),
          +[t(DP,rdf:type,rdf:property)]; DP.

sop(OP) -> 
          t(OP,rdf:type,owl:objectProperty),
          +[t(OP,rdf:type,rdf:property)];
                                          OP.

sop(rdfs:seeAlso) -> true; rdfs:seeAlso .
sop(rdfs:isDefinedBy) -> true; rdfs:isDefinedBy.
cop(OP) -> 
          note(prop)/(+[t(OP,rdf:type,owl:objectProperty)]),
          +[t(OP,rdf:type,rdf:property)]; OP.

tp(OP) -> 
          note(prop)/(+[t(OP,rdf:type,owl:objectProperty)]),
          +[t(OP,rdf:type,rdf:property)]; OP.
text -> heading(1,'Literals').
dataLiteral(lexicalForm,language,dt)->true; lexicalForm@language^^x(dt).
dataLiteral(lexicalForm,dt)->true; lexicalForm^^x(dt).
dataLiteral(lexicalForm,language)->true; lexicalForm@language.
dataLiteral(lexicalForm)->true; lexicalForm.
         

text -> heading(1,'Ontologies').

ontology( { directive } ) -> 
               {note(directive)/x(directive)}.


text2 -> heading(1,'Header and Annotations').
header(o,{metaproperty},{annotation}) -> 
          t(x(o),rdf:type,owl:ontology),
          {x(x(o),metaproperty)},
          {x(x(o),annotation)}.

D+metaproperty( m, o ) -> 
          t(D,x(m),x(o) ).

D + annotation( sdp, dataLiteral ) -> 
          t(D,x(sdp),x(dataLiteral)).
D + annotation( sop, individual ) ->
          t(D,x(sop),x(individual) ).

text -> heading(1,individuals).
text -> heading(2,'Named Individuals').
text -> 'The interaction between these two individual rules make it obligatory 
to declare at least one type for a named individual.
The default is <code>owl:thing</code>'.

individual( i, type( type-1 ), {type( type-i )}, 
            {propertyValue}, {individualComparision} ) ->
       t(x(i),rdf:type,x(type-1)),
       {t(x(i),rdf:type,x(type-i))},
       {x(x(i),propertyValue)},
       {x(x(i),individualComparison)};            x(i).


individual( i, {type( type )}, 
            {propertyValue}, {individualComparision} ) ->
       t(x(i),rdf:type,owl:thing),
       {t(x(i),rdf:type,x(type))},
       {x(x(i),propertyValue)},
       {x(x(i),individualComparison)};            x(i).

text -> heading(2,'Unnamed Individuals').

 
individual(   type( type-1 ), {type( type-i )}, 
            {propertyValue}, {individualComparision} ) ->
       t(blank,rdf:type,x(type-1)),
       {t(blank,rdf:type,x(type-i))},
       {x(blank,propertyValue)},
       {x(x(i),individualComparison)};            blank.


individual(  {type( type )}, 
            {propertyValue}, {individualComparision} ) ->
       t(blank,rdf:type,owl:thing),
       {t(blank,rdf:type,x(type))},
       {x(blank,propertyValue)},
       {x(x(i),individualComparison)};            blank.   

text2 -> heading(1,'Property Values').

D+propertyValue(sop,individual) ->
       t(D,x(sop),x(individual)).
D+propertyValue(cop,individual) ->
       t(D,x(cop),x(individual)).
D+propertyValue(tp,individual) ->
       t(D,x(tp),x(individual)).
D+propertyValue(sdp,dataLiteral) ->
       t(D,x(sdp),x(dataLiteral)).
D+propertyValue(cdp,dataLiteral) ->
       t(D,x(cdp),x(dataLiteral)).


text2 -> heading(1,'Comparison of Individuals').

D+sameIndividual(individual)->
   t(D,owl:sameIndividualAs,x(individual)).
D+differentIndividuals(individual)->
   t(D,owl:differentFrom,x(individual)).

text -> heading(1,'Classes').


class(c, +partial, {annotation}, {super(type)},
     { classComparison }
   ) ->
   t(x(c),rdf:type,owl:class),
   +[t(x(c),rdf:type,rdfs:class)],
   {x(x(c),annotation)},
   {t(x(c),rdfs:subClassOf,x(type))},
   {x(x(c),classComparison)};     x(c).

class(c, +complete, {annotation}, {super(type)},
     { classComparison }
   ) ->
   t(x(c),rdf:type,owl:class),
   +[t(x(c),rdf:type,rdfs:class)],
   {x(x(c),annotation)},
   t(x(c),owl:intersectionOf,x(seq({super(type)}))),
   {x(x(c),classComparison)};     x(c).

class(c, +complete, {annotation}, super(unionOf({description}, {classComparison-2})),
     { classComparison-1 }
   ) ->
   t(x(c),rdf:type,owl:class),
   +[t(x(c),rdf:type,rdfs:class)],
   {x(x(c),annotation)},
   t(x(c),owl:unionOf,x(seq({description}))),
   {x(x(c),classComparison-2)},
   {x(x(c),classComparison-1)};     x(c).


class(c, +complete, {annotation}, super(complementOf(description, {classComparison-2})),
     { classComparison-1 }
   ) ->
   t(x(c),rdf:type,owl:class),
   +[t(x(c),rdf:type,rdfs:class)],
   {x(x(c),annotation)},
   t(x(c),owl:complementOf,t(description)),
   {x(x(c),classComparison-2)},
   {x(x(c),classComparison-1)};     x(c).
      
text -> heading(2,'Enumerated Classes').
enumeratedClass(c, { annotation }, {i}, { classComparison } )->
   t(x(c),rdf:type,owl:class),
   +[t(x(c),rdf:type,rdfs:class)],
   {x(x(c),annotation)},
   t(x(c),owl:oneOf,x(seq({i}))),
   {x(x(c),classComparison)};     x(c).


text2 -> heading(1,'Comparison of Classes').

D+sameClassAs(description)->
   t(D,owl:sameClassAs,x(description)).
D+disjointWith(description)->
   t(D,owl:disjointWith,x(description)).
D+subClassOf(description)->
   t(D,rdfs:subClassOf,x(description)).

text -> heading(1,'Restrctions').

text -> heading(2,'Restrictions on Datatype Properties').


restriction( cdp, allValuesFrom( dataRange ) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cdp)),
   t(blank,owl:allValuesFrom,x(dataRange));  
                                        blank.

restriction( cdp, someValuesFrom( dataRange ) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cdp)),
   t(blank,owl:someValuesFrom,x(dataRange));  
                                        blank.    

restriction( cdp, cardinality(smallInt) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cdp)),
   t(blank,owl:cardinality,x(smallInt));  
                                        blank.    
restriction( cdp, minCardinality(smallInt) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cdp)),
   t(blank,owl:minCardinality,x(smallInt));  
                                        blank.     
restriction( cdp, maxCardinality(smallInt) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cdp)),
   t(blank,owl:maxCardinality,x(smallInt));  
                                        blank.     
restriction( cdp, value(dataLiteral) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cdp)),
   t(blank,owl:hasValue,x(dataLiteral));  
                                        blank.  


text -> heading(2,'Restrictions on Object Properties').

restriction( cop, allValuesFrom( classInRestriction ) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cop)),
   t(blank,owl:allValuesFrom,x(classInRestriction));  
                                        blank.

restriction( cop, someValuesFrom( classInRestriction ) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cop)),
   t(blank,owl:someValuesFrom,x(classInRestriction));  
                                        blank.    

restriction( cop, cardinality(smallInt) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cop)),
   t(blank,owl:cardinality,x(smallInt));  
                                        blank.    
restriction( cop, minCardinality(smallInt) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cop)),
   t(blank,owl:minCardinality,x(smallInt));  
                                        blank.     
restriction( cop, maxCardinality(smallInt) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cop)),
   t(blank,owl:maxCardinality,x(smallInt));  
                                        blank.      
restriction( cop, value(i) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(cop)),
   t(blank,owl:hasValue,x(i));  
                                        blank.  

text -> heading(2,'Restrictions on Transitive Properties').


restriction( tp, allValuesFrom( classInRestriction ) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(tp)),
   t(blank,owl:allValuesFrom,x(classInRestriction));  
                                        blank.

restriction( tp, someValuesFrom( classInRestriction ) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(tp)),
   t(blank,owl:someValuesFrom,x(classInRestriction));  
                                        blank.     
restriction( tp, value(i) ) ->
   t(blank,rdf:type,owl:restriction),
   t(blank,owl:onProperty,x(tp)),
   t(blank,owl:hasValue,x(i));  
                                        blank.  

text -> heading(2,'Numbers in Cardinalities').
0 -> true; '"0"'^^xsd:nonNegativeInteger.
1 -> true; '"1"'^^xsd:nonNegativeInteger.
naturalNumber -> true; naturalNumber^^xsd:nonNegativeInteger.


text -> heading(1,'Property Axioms' ).
datatypeProperty( sdp-1, { annotation },
                    { super(sdp-2) } )  ->
   t(x(sdp-1),rdf:type,owl:datatypeProperty),
  +[t(x(sdp-1),rdf:type,rdf:property)],
   { x(x(sdp-1),annotation) },
   { t(x(sdp-1),rdfs:subPropertyOf,x(sdp-2)) }.
  
datatypeProperty( cdp-1, { annotation },
                    { super(sdp) },
                    { super(cdp-2) },
                    { domain(classInRestriction) },
                    { range(dataRange) }, [+functional] ) ->
   t(x(cdp-1),rdf:type,owl:datatypeProperty),
  +[t(x(cdp-1),rdf:type,rdf:property)],
   { x(x(cdp-1),annotation) },
   { t(x(cdp-1),rdfs:subPropertyOf,x(sdp)) },
   { t(x(cdp-1),rdfs:subPropertyOf,x(cdp-2)) },
   { t(x(cdp-1), rdfs:domain, x(classInRestriction)) },
   { t(x(cdp-1), rdfs:range, x(dataRange) ) },
   [ t(x(cdp-1), rdf:type, owl:functionalProperty) ].

individualProperty( sop-1, { annotation },
                    { super(sop-2) } ) ->
   t(x(sop-1),rdf:type,owl:objectProperty),
  +[t(x(sop-1),rdf:type,rdf:property)],
   { x(x(sop-1),annotation) },
   { t(x(sop-1),rdfs:subPropertyOf,x(sop-2)) }.


individualProperty( cop-1, { annotation },
                    { super(sop) },
                    { super(cop-2) },
                    { super(tp) },
                    { domain(classInRestriction-1) },
                    { range(classInRestriction-2) }, 
                    { inverse(cop-3) },
                    [+functional], 
                    [+inverseFunctional], 
                    [+symmetric]) ->
   note(prop)/(+[t(x(cop-1),rdf:type,owl:objectProperty)]),
  +[t(x(cop-1),rdf:type,rdf:property)],
   { x(x(cop-1),annotation) },
   { t(x(cop-1),rdfs:subPropertyOf,x(sop)) },
   { t(x(cop-1),rdfs:subPropertyOf,x(cop-2)) },
   { t(x(cop-1),rdfs:subPropertyOf,x(tp)) },
   { t(x(cop-1), rdfs:domain, x(classInRestriction-1)) },
   { t(x(cop-1), rdfs:range, x(classInRestriction-2)) },
   { t(x(cop-1), owl:inverseOf, x(cop-3)) },
   [t(x(cop-1),rdf:type,owl:functionalProperty)],
   [t(x(cop-1),rdf:type,owl:inverseFunctionalProperty)],
   [t(x(cop-1),rdf:type,owl:symmetricProperty)].

individualProperty( tp-1, { annotation },
                    { super(sop) },
                    { super(tp-2) },
                    { domain(classInRestriction-1) },
                    { range(classInRestriction-2) }, 
                    { inverse(tp-3) },
                    [+transitive]) ->
   note(prop)/( +[t(x(tp-1),rdf:type,owl:objectProperty)]),
  +[t(x(tp-1),rdf:type,rdf:property)],
   { x(x(tp-1),annotation) },
   { t(x(tp-1),rdfs:subPropertyOf,x(sop)) },
   { t(x(tp-1),rdfs:subPropertyOf,x(tp-2)) },
   { t(x(tp-1), rdfs:domain, x(classInRestriction-1)) },
   { t(x(tp-1), rdfs:range, x(classInRestriction-2)) },
   { t(x(tp-1), owl:inverseOf, x(tp-3)) },
   [t(x(cop-1),rdf:type,owl:transitiveProperty)].

text -> heading(1,'Equivalent Properties').
equivalentProperties(sop-1,sop-2,{sop-3}) ->
   t(x(sop-1),owl:samePropertyAs,x(sop-2)),
   {t(x(sop-1),owl:samePropertyAs,x(sop-3))}.

equivalentProperties(tp-1,tp-2,{tp-3}) ->
   t(x(tp-1),owl:samePropertyAs,x(tp-2)),
   {t(x(tp-1),owl:samePropertyAs,x(tp-3))}.

equivalentProperties(cop-1,cop-2,{cop-3}) ->
   t(x(cop-1),owl:samePropertyAs,x(cop-2)),
   {t(x(cop-1),owl:samePropertyAs,x(cop-3))}.


equivalentProperties(sdp-1,sdp-2,{sdp-3}) ->
   t(x(sdp-1),owl:samePropertyAs,x(sdp-2)),
   {t(x(sdp-1),owl:samePropertyAs,x(sdp-3))}.

equivalentProperties(cdp-1,cdp-2,{cdp-3}) ->
   t(x(cdp-1),owl:samePropertyAs,x(cdp-2)),
   {t(x(cdp-1),owl:samePropertyAs,x(cdp-3))}.

datatypeDeclaration(dt) -> x(dt).
text -> heading(1,'Descriptions').



intersectionOf({description}, {classComparison}) ->
   t(blank,rdf:type,owl:class),
   t(blank,owl:intersectionOf,x(seq({description}))),
   {x(blank,classComparison)};     blank.

unionOf({description}, {classComparison}) ->
   t(blank,rdf:type,owl:class),
   t(blank,owl:unionOf,x(seq({description}))),
   {x(blank,classComparison)};     blank.

complementOf(description, {classComparison}) ->
   t(blank,rdf:type,owl:class),
   t(blank,owl:complementOf,x(description)),
   {x(blank,classComparison)};     blank.

oneOf({i}, {classComparison}) ->
   t(blank,rdf:type,owl:class),
   t(blank,owl:oneOf,x(seq({i}))),
   {x(blank,classComparison)};     blank.

text -> heading(1,'DataRanges').

oneOf( {dataLiteral} ) ->
   t(blank,rdf:type,owl:dataRange),
   t(blank,owl:oneOf,x(seq({dataLiteral})));     blank.
 
dataRange( dr, {annotation}, {dataLiteral} ) ->
   t(x(dr),rdf:type,owl:dataRange),
   {x(x(dr),annotation)},
   t(x(dr),owl:oneOf,x(seq({dataLiteral})));     x(dr).
dataRange( dr, +declaration, {annotation} ) ->
   t(x(dr),rdf:type,owl:dataRange),
   {x(x(dr),annotation)};     x(dr).
   


